function [minBIC, b] = get_space_bearing_glmfit_BIC(pos, spkb, nh)
%get_space_bearing_glmfit_BIC Fit space+bearing model to the data and get BIC
%   minBIC = get_space_bearing_glmfit_BIC(pos, spkb, nh)
%
%   Input:
%       pos(mat) - [ts, x, y, bearing]
%                   ts: timestamp in seconds, x,y: spatial coordinates,
%                   bearing: head direction/egocentric bearing in radian [0, 2*pi)
%       spkb(vec) - binned spike count vector, has to be the same length as pos
%       nh(num) - number of head direction/egocentric bearing components,
%                 as pairs of sine and cosine
%
%   Output:
%       minBIC(num) - minimum BIC
%       b(vec) - parameters
%
%   Notes: deviance(dev) or likelihood ratio statistic, is defined as:
%   2(log(L_sat) - log(L_model)), here this code uses the real BIC
%
%   by Cheng Wang (cwchengwang@gmail.com)

% spatial component of the data matrix
[atheta, aR] = prepare_pos(pos);
zerns = zernike_helper(atheta, aR, 32); % at most 32 zernike basis

% head direction component of the data matrix
bearingdata = bearing_helper(pos(:, 4), nh); % nh paris of sine and cosine

b = cell(size(zerns, 2) - 4, 1); % estimated parameters
Dev = zeros(1, size(zerns, 2) - 4) + nan; % deviance
for i = 5:size(zerns, 2)
    [b{i-4}, Dev(i-4)] = glmfit([zerns(:, 1:i), bearingdata], spkb, 'poisson');
end

% select the one with the best (minimal) Dev, deviance
k = (5:size(zerns, 2)) + nh*2; % number of parameters, at least 5 Zernike
Dev = Dev + k * log(size(pos, 1)); % deviance
[~, idx] = nanmin(Dev);
b = b{idx}; 
nz = idx + 5 - 1; % number of zernike (spatial components)

% get the BIC
pfit = glmval(b, [zerns(:, 1:nz), bearingdata], 'log');
LL = sum(log(poisspdf(spkb,pfit)));
minBIC = -2*LL + length(b) * log(size(pos, 1));

end
% =========================================================================
% helper functions
% =========================================================================
function [theta, R] = prepare_pos(pos)
% Rescale the pos to ensure max(R) <= 1
x = pos(:, 2); y = pos(:, 3);
[theta, R] = cart2pol(x, y);
maxR = max(R);
R = R/maxR;
end

function zerns = zernike_helper(theta, R, nz)
% Get spatial components, i.e., zernike function of space.
% zernfun2.m from https://www.mathworks.com/matlabcentral/fileexchange/7687-zernike-polynomials
zerns = zeros(length(R), nz);
for i = 1:nz
    zerns(:, i) = zernfun2(i, R, theta);
end
end

function data = bearing_helper(bearing, nh)
% Prepare the head direction component or egocentric bearing component
data = zeros(length(bearing), nh*2);
for i = 1:nh
    data(:, i*2 - 1) = sin(i*bearing);
    data(:, i*2) = cos(i*bearing);
end
end